unit SelTextControl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TSelTextControl = class(TCustomControl)
  private
    FSelTextColor,
    FSelBrushColor: TColor;
    FStrings: TStringList;
    procedure SetSelBrushColor(const Value: TColor);
    procedure SetSelTextColor(const Value: TColor);
    procedure SetStrings(const Value: TStringList);
    { Dclarations prives }
  protected
    { Dclarations protges }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);override;
  public
    { Dclarations publiques }
    SelLine, SelStartPos, SelEndPos: shortInt;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Strings: TStringList read FStrings write SetStrings;
    property SelTextColor: TColor read FSelTextColor write SetSelTextColor;
    property SelBrushColor: TColor read FSelBrushColor write SetSelBrushColor;
    procedure Paint; override;
  published
    { Dclarations publies }
  end;

implementation

{ TCustomControl1 }

const
   pas: byte = 15; //line height
   margin: byte = 10;

constructor TSelTextControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Strings := TStringList.Create;
  SelLine := -1;//no selected line
end;

destructor TSelTextControl.Destroy;
begin
  Strings.Free;
  inherited Destroy;
end;

procedure TSelTextControl.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
   SelLine := (Y-margin) div pas;//calculates the selected line #
   Invalidate;//redraws after SelLine calculation
   inherited MouseDown(Button, Shift, X, Y);
end;

procedure TSelTextControl.Paint;
var
   r :TRect;
   buffer: TBitmap;
   canv: TCanvas;
   s, s1: string;
   i: byte;
   x, y: integer;
begin

   r := Canvas.ClipRect;
   buffer := TBitmap.Create;
   buffer.Width := r.Right-r.Left+1;
   buffer.Height := r.Bottom-r.Top+1;
   canv := buffer.Canvas;
   canv.Brush.Style := bsClear;

   for i := 0 to FStrings.Count -1 do begin
   x := margin;
   y := margin + i*pas;//position the text for each FStrings item
   if i = SelLine then begin //if current line is the one selected
      //draws before selection
      canv.Font.Color := clBlack;
      s := Copy(FStrings[i], 1, SelStartPos-1);
      canv.TextOut(x, y, s);
      //draws selection
      canv.Brush.Style := bsSolid;
      canv.Brush.Color := SelBrushColor;
      canv.Font.Color := SelTextColor;
      s1 := Copy(FStrings[i], SelStartPos, SelEndPos-SelStartPos);
      canv.TextOut(x+canv.TextWidth(s), y, s1);
      //draws after selection
      canv.Font.Color := clBlack;
      canv.Brush.Style := bsClear;
      canv.TextOut(x+canv.TextWidth(s+s1), y,
         Copy(FStrings[i], SelEndPos, Length(FStrings[i])));
   end else begin //if current line is not the one selected
      canv.Font.Color := clBlack;
      canv.TextOut(x, y, FStrings[i]);
   end;
   end;
   Canvas.Draw(Canvas.ClipRect.Left, Canvas.ClipRect.Top, buffer);
   buffer.Free;
end;

procedure TSelTextControl.SetSelBrushColor(const Value: TColor);
begin
  FSelBrushColor := Value;
end;

procedure TSelTextControl.SetSelTextColor(const Value: TColor);
begin
  FSelTextColor := Value;
end;

procedure TSelTextControl.SetStrings(const Value: TStringList);
begin
  FStrings := Value;
end;

end.
